import java.util.*;

abstract public class M865Graph implements Cloneable {

	// This abstract definition requires that a graph contain a set
	// of vertices and a set of edges.  Both sets must be sorted sets.
	// Any concrete implementation of this class must initialize both
	// the vertexSet and the edgeList to a concrete - though possibly
	// empty sorted set.
	protected SortedSet vertexSet;		// the set of vertices
	protected SortedSet edgeList;			// the set of edges
	protected SortedMap neighborhoods;	// the map between vertices and their set of neighbors

	public M865Graph(M865Graph graph) {
		this.vertexSet = graph.getVertexSet();
		this.edgeList = graph.getEdgeList();
		this.neighborhoods = null;
	}

	public M865Graph(SortedSet vertexSet, SortedSet edgeList) {
		this.vertexSet = vertexSet;
		this.edgeList = edgeList;
		this.neighborhoods = null;
	}


	public SortedSet getVertexSet() {
		return vertexSet;
	}
	
	public SortedSet getEdgeList() {
		return edgeList;
	}

	public int getNumberOfVertices() {
		return vertexSet.size();
	}

	public int getNumberOfEdges() {
		return edgeList.size();
	}

	public boolean containsVertex(M865Vertex v) {
		return vertexSet.contains(v);
	}

	public boolean addVertex(M865Vertex v) {
		if (containsVertex(v)) {
			return false;
		} else {
			vertexSet.add(v);
			return true;
		}
	}

	public boolean removeVertex(M865Vertex v) {
		if (containsVertex(v)) {
			vertexSet.remove(v);
			Iterator iter = edgeList.iterator();
			while (iter.hasNext()) {
				M865Edge e = (M865Edge)iter.next();
				M865Vertex s = e.getSource();
				M865Vertex t = e.getTarget();
				if (v.equals(s) || v.equals(t)) {
					iter.remove();
				}
			}
			return true;
		} else {
			return false;
		}
	}

	public boolean containsEdge(M865Edge e) {
		return edgeList.contains(e);
	}

	public boolean addEdge(M865Edge e) {
		if (containsEdge(e)) {
			return false;
		} else {
			vertexSet.add((M865Vertex)e.getSource());
			vertexSet.add((M865Vertex)e.getTarget());
			edgeList.add(e);
			return true;
		}
	}

	public boolean removeEdge(M865Edge e) {
		if (containsEdge(e)) {
			edgeList.remove(e);
			return true;
		} else {
			return false;
		}
	}

	public boolean equals(Object g) {
		if (g instanceof M865Graph) {
			return (vertexSet.equals(((M865Graph)g).getVertexSet()) &&
					  edgeList.equals(((M865Graph)g).getEdgeList()));
		} else {
			throw new ClassCastException("Not a graph");
		}
	}

	public int hashCode() {
		long hash = (vertexSet.hashCode() + edgeList.hashCode()) % Integer.MAX_VALUE;
		return (int)hash;
	}


	abstract public int getNumberOfComponents();
	
	public boolean isConnected() {
		return (getNumberOfComponents() <= 1);
	}

	abstract public boolean hasCycles();
	
	abstract public M865Graph getSpanningTree();
	
	abstract public int degreeOf(M865Vertex vertex);
	
	abstract public SortedSet neighborhoodOf(M865Vertex vertex);

	public void printNeighborhoods() {
		System.out.println("Neighborhoods");
		Iterator iter = vertexSet.iterator();
		while (iter.hasNext()) {
			M865Vertex v = (M865Vertex)iter.next();
			System.out.println(v + ": " + neighborhoodOf(v));
		}
	}

	public String toString() {
		String s = "M865Graph (\n   Vertex Set: ";
		s += vertexSet.toString() + "\n   Edge List: ";
		s += edgeList.toString() + ")";
		return s;
	}	
}	
